@@ -72,7 +72,7 @@ class Client:
             created sessions are loaded.
     """
 
-    INVITE_LINK_RE = re.compile(r"^(?:https?:\/\/)?t\.me\/joinchat\/(.+)$")
+    INVITE_LINK_RE = re.compile(r"^(?:https?://)?t\.me/joinchat/(.+)$")
     DIALOGS_AT_ONCE = 100
 
     def __init__(self, session_name: str, test_mode: bool = False):
@@ -602,12 +602,12 @@ class Client:
                         peer=self.resolve_peer(chat_id),
                         media=types.InputMediaUploadedPhoto(
                             file=file,
-                            caption=caption,
                             ttl_seconds=ttl_seconds
                         ),
                         silent=disable_notification or None,
                         reply_to_msg_id=reply_to_message_id,
-                        random_id=self.rnd_id()
+                        random_id=self.rnd_id(),
+                        **self.markdown.parse(caption)
                     )
                 )
             except FilePartMissing as e:
@@ -673,7 +673,6 @@ class Client:
                         media=types.InputMediaUploadedDocument(
                             mime_type=mimetypes.types_map.get("." + audio.split(".")[-1], "audio/mpeg"),
                             file=file,
-                            caption=caption,
                             attributes=[
                                 types.DocumentAttributeAudio(
                                     duration=duration,
@@ -685,7 +684,8 @@ class Client:
                         ),
                         silent=disable_notification or None,
                         reply_to_msg_id=reply_to_message_id,
-                        random_id=self.rnd_id()
+                        random_id=self.rnd_id(),
+                        **self.markdown.parse(caption)
                     )
                 )
             except FilePartMissing as e:
@@ -737,14 +737,14 @@ class Client:
                         media=types.InputMediaUploadedDocument(
                             mime_type=mimetypes.types_map.get("." + document.split(".")[-1], "text/plain"),
                             file=file,
-                            caption=caption,
                             attributes=[
                                 types.DocumentAttributeFilename(os.path.basename(document))
                             ]
                         ),
                         silent=disable_notification or None,
                         reply_to_msg_id=reply_to_message_id,
-                        random_id=self.rnd_id()
+                        random_id=self.rnd_id(),
+                        **self.markdown.parse(caption)
                     )
                 )
             except FilePartMissing as e:
@@ -808,7 +808,6 @@ class Client:
                         media=types.InputMediaUploadedDocument(
                             mime_type=mimetypes.types_map[".mp4"],
                             file=file,
-                            caption=caption,
                             attributes=[
                                 types.DocumentAttributeVideo(
                                     duration=duration,
@@ -819,7 +818,8 @@ class Client:
                         ),
                         silent=disable_notification or None,
                         reply_to_msg_id=reply_to_message_id,
-                        random_id=self.rnd_id()
+                        random_id=self.rnd_id(),
+                        **self.markdown.parse(caption)
                     )
                 )
             except FilePartMissing as e:
@@ -875,7 +875,6 @@ class Client:
                         media=types.InputMediaUploadedDocument(
                             mime_type=mimetypes.types_map.get("." + voice.split(".")[-1], "audio/mpeg"),
                             file=file,
-                            caption=caption,
                             attributes=[
                                 types.DocumentAttributeAudio(
                                     voice=True,
@@ -885,7 +884,8 @@ class Client:
                         ),
                         silent=disable_notification or None,
                         reply_to_msg_id=reply_to_message_id,
-                        random_id=self.rnd_id()
+                        random_id=self.rnd_id(),
+                        **self.markdown.parse(caption)
                     )
                 )
             except FilePartMissing as e:
@@ -941,7 +941,6 @@ class Client:
                         media=types.InputMediaUploadedDocument(
                             mime_type=mimetypes.types_map[".mp4"],
                             file=file,
-                            caption="",
                             attributes=[
                                 types.DocumentAttributeVideo(
                                     round_message=True,
@@ -951,6 +950,7 @@ class Client:
                                 )
                             ]
                         ),
+                        message="",
                         silent=disable_notification or None,
                         reply_to_msg_id=reply_to_message_id,
                         random_id=self.rnd_id()
@@ -1003,6 +1003,7 @@ class Client:
                         longitude
                     )
                 ),
+                message="",
                 silent=disable_notification or None,
                 reply_to_msg_id=reply_to_message_id,
                 random_id=self.rnd_id()
@@ -1068,6 +1069,7 @@ class Client:
                     venue_id=foursquare_id,
                     venue_type=""
                 ),
+                message="",
                 silent=disable_notification or None,
                 reply_to_msg_id=reply_to_message_id,
                 random_id=self.rnd_id()
@@ -1119,6 +1121,7 @@ class Client:
                     first_name,
                     last_name
                 ),
+                message="",
                 silent=disable_notification or None,
                 reply_to_msg_id=reply_to_message_id,
                 random_id=self.rnd_id()
@@ -1244,7 +1247,7 @@ class Client:
             functions.messages.EditMessage(
                 peer=self.resolve_peer(chat_id),
                 id=message_id,
-                message=caption
+                **self.markdown.parse(caption)
             )
         )
 
@@ -1637,3 +1640,115 @@ class Client:
                     return channel_full.exported_invite.link
                 else:
                     raise ChatAdminRequired
+
+    def enable_cloud_password(self, password: str, hint: str = "", email: str = ""):
+        """Use this method to enable the Two-Step Verification security feature (Cloud Password) on your account.
+
+        This password will be asked when you log in on a new device in addition to the SMS code.
+
+        Args:
+            password (:obj:`str`):
+                Your password.
+
+            hint (:obj:`str`, optional):
+                A password hint.
+
+            email (:obj:`str`, optional):
+                Recovery e-mail.
+
+        Returns:
+            True on success, False otherwise.
+
+        Raises:
+            :class:`pyrogram.Error`
+        """
+        r = self.send(functions.account.GetPassword())
+
+        if isinstance(r, types.account.NoPassword):
+            salt = r.new_salt + os.urandom(8)
+            password_hash = sha256(salt + password.encode() + salt).digest()
+
+            return self.send(
+                functions.account.UpdatePasswordSettings(
+                    current_password_hash=salt,
+                    new_settings=types.account.PasswordInputSettings(
+                        new_salt=salt,
+                        new_password_hash=password_hash,
+                        hint=hint,
+                        email=email
+                    )
+                )
+            )
+        else:
+            return False
+
+    def change_cloud_password(self, current_password: str, new_password: str, new_hint: str = ""):
+        """Use this method to change your Two-Step Verification password (Cloud Password) with a new one.
+
+        Args:
+            current_password (:obj:`str`):
+                Your current password.
+
+            new_password (:obj:`str`):
+                Your new password.
+
+            new_hint (:obj:`str`, optional):
+                A new password hint.
+
+        Returns:
+            True on success, False otherwise.
+
+        Raises:
+            :class:`pyrogram.Error`
+        """
+        r = self.send(functions.account.GetPassword())
+
+        if isinstance(r, types.account.Password):
+            current_password_hash = sha256(r.current_salt + current_password.encode() + r.current_salt).digest()
+
+            new_salt = r.new_salt + os.urandom(8)
+            new_password_hash = sha256(new_salt + new_password.encode() + new_salt).digest()
+
+            return self.send(
+                functions.account.UpdatePasswordSettings(
+                    current_password_hash=current_password_hash,
+                    new_settings=types.account.PasswordInputSettings(
+                        new_salt=new_salt,
+                        new_password_hash=new_password_hash,
+                        hint=new_hint
+                    )
+                )
+            )
+        else:
+            return False
+
+    def remove_cloud_password(self, password: str):
+        """Use this method to turn off the Two-Step Verification security feature (Cloud Password) on your account.
+
+        Args:
+            password (:obj:`str`):
+                Your current password.
+
+        Returns:
+            True on success, False otherwise.
+
+        Raises:
+            :class:`pyrogram.Error`
+        """
+        r = self.send(functions.account.GetPassword())
+
+        if isinstance(r, types.account.Password):
+            password_hash = sha256(r.current_salt + password.encode() + r.current_salt).digest()
+
+            return self.send(
+                functions.account.UpdatePasswordSettings(
+                    current_password_hash=password_hash,
+                    new_settings=types.account.PasswordInputSettings(
+                        new_salt=b"",
+                        new_password_hash=b"",
+                        hint=""
+                    )
+                )
+            )
+        else:
+            return False
